/* SPDX-License-Identifier: GPL-2.0 */
/*RTL8188F PHY Parameters*/
/* 
[Caution] 
  Since 01/Aug/2015, the commit rules will be simplified. 
  You do not need to fill up the version.h anymore, 
  only the maintenance supervisor fills it before formal release.
*/
#define	RELEASE_DATE_8188F		20160624
#define	COMMIT_BY_8188F			"BB_David"
#define	RELEASE_VERSION_8188F		31
